#include "notice.h"

// Submodules
#include "http_server/http_request.h"
#include "ffdb/fs_list.h"
#include "model/notification.h"
#include "collections/array.h"

// Model
#include "status.h"

// Stdlib
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void show_notifications( struct http_request* req, struct notification** ns, int count )
{
	http_request_send_headers( req, 200, "application/json", true );
	FILE* f = http_request_get_response_body( req );

	fprintf( f, "[" );
	for( int i = 0; i < count; ++i ) {
		if( i > 0 ) {
			fprintf( f, "," );
		}
		notification_write_as_json(ns[i],f);
	}
	fprintf( f, "]" );
}

bool handle_notifications( struct http_request* req )
{
	int note_count = fs_list_get("data/notices/HEAD");

	struct {
		struct notification** items;
		int count;
	} notes;
	memset(&notes,0,sizeof(notes));

	int limit = 32;
	bool hide_muted = true;

	// Handle query parameters
	if( http_request_route( req, "?" ) ) {
		const char* key;
		while( key = http_request_route_query_key(req) ) {
			if( 0 == strcmp(key,"limit") ) {
				int new_limit;
				sscanf(http_request_route_query_value(req),"%d",&new_limit);
				if( new_limit < limit ) {
					limit = new_limit;
				}
			} else if( 0 == strcmp(key,"with_muted") ) {
				hide_muted = false;
			} else if( 0 == strcmp(key,"since_id") ) {
				int since_id = 0;
				sscanf(http_request_route_query_value(req),"%d",&since_id);
				if( since_id <= note_count ) {
					limit = note_count - since_id;
				} else {
					limit = 0;
				}
			}
		}
	}

	for( int i = 0; (i < limit) && (i < note_count); ++i ) {
		int note_id = note_count - i;
		struct notification* n = notification_from_id(note_id);
		bool include = true;

		// Apply filters
		if( !n ) { include = false; }
		if( n ) {
		}

		if( include ) { // TODO: filter notes here
			array_append( &notes, sizeof(void*), &n );
		} else {
			printf( "Failed to load notice #%d\n", note_id );
			notification_free(n);
		}
	}

	show_notifications( req, notes.items, notes.count );
	for( int i = 0; i < notes.count; ++i ) {
		notification_free(notes.items[i]);
	}
	free(notes.items);

	return true;
}

