#include "account.h"

#include "json/json.h"
#include "json/layout.h"

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

static struct json_enum ap_account_type_enum[] = {
	{ "Person", apacct_Person },
	{ NULL, 0 },
};

static struct json_object_field public_key_layout[] = {
	{ "id",				offsetof(struct ap_crypto_key,id),		true,  &json_field_string },
	{ "owner",			offsetof(struct ap_crypto_key,owner),	true,  &json_field_string },
	{ "publicKeyPem",		offsetof(struct ap_crypto_key,pem),	true,  &json_field_string },
	{ NULL, 0 },
};

static struct json_object_field icon_layout[] = {
	{ "url",				offsetof(struct ap_account,avatar),			true,  &json_field_string },
	{ NULL },
};

static struct json_object_field image_layout[] = {
	{ "url",				offsetof(struct ap_account,banner),			true,  &json_field_string },
	{ NULL },
};

static struct json_object_field ap_account_layout[] = {
	{ "id",				offsetof(struct ap_account,id),				true,  &json_field_string },
	{ "type",				offsetof(struct ap_account,type),				true,  &json_field_enum, ap_account_type_enum },
	{ "url",				offsetof(struct ap_account,url),				true,  &json_field_string },
	{ "name",				offsetof(struct ap_account,name),				true,  &json_field_string },
	{ "preferredUsername",	offsetof(struct ap_account,preferredUsername),	true,  &json_field_string },
	{ "inbox",			offsetof(struct ap_account,inbox),				true,  &json_field_string },
	{ "summary",			offsetof(struct ap_account,summary),			true,  &json_field_string },
	{ "featured",			offsetof(struct ap_account,featured),			false, &json_field_string },
	{ "followers",			offsetof(struct ap_account,followers),			false, &json_field_string },
	{ "following",			offsetof(struct ap_account,following),			false, &json_field_string },
	{ "publicKey",			offsetof(struct ap_account,public_key),			false, &json_field_object_composite, public_key_layout },
	{ "discoverable",		offsetof(struct ap_account,discoverable),		false, &json_field_bool },
	{ "icon",				0,										false, &json_field_object_composite, icon_layout },
	{ "image",			0,										false, &json_field_object_composite, image_layout },

	{ NULL },
};

struct ap_account* ap_account_from_file( const char* filename )
{
	struct ap_account* acc = malloc(sizeof(struct ap_account));
	memset(acc,0,sizeof(struct ap_account));

	if( !json_read_object_layout_from_file( filename, ap_account_layout, acc ) ) {
		ap_account_free(acc);
		return NULL;
	}

	return acc;
}

void ap_account_free( struct ap_account* acc )
{
	if( !acc ) { return; }

	free(acc->id);
	free(acc->url);

	free(acc);
}
void ap_account_debug_dump( struct ap_account* acc )
{
	FILE* f = stdout;
	#include "src/model/ap/account.debug_dump.txt.inc"
}

